#!/usr/bin/env python

# Copyright (C) 2009 FoAM vzw
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Inputs haplotype networks and performs prim's algorithm on them to 
# calculate the minimum spanning tree. This tree is then run through 
# a force directed algorithm in order to provide suitable figures 
# which visualise distance and relatedness.
#
# dave@fo.am

version = "0.7"
midx=250
midy=240

import platform; 

if int(platform.python_version_tuple()[0])>2:
	from tkinter import *
	from tkinter.filedialog import *
	from tkinter.messagebox import *
else:  
	from Tkinter import *
	from tkFileDialog import *
	from tkMessageBox import *
	
import os,random,math,copy,string

def write_out(f,v):
    f.write(str(v)+"\n")

def read_in(f):
    return f.readline()[0:-1];

	
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# an undirected, weighted graph

class graph:
    # edges consists of a list of lists:
    # [[node1,node2,weight],[node1,node2,weight]...]
    # the graph also internally keeps track of 
    # the unique vertices
    def __init__(self,edges):
        self.edges = edges
        self.update_vertices()
        self.root=""

    def serialise_save(self,f):
        version=1
        write_out(f,"graph")
        write_out(f,version)
        write_out(f,self.root)
        write_out(f,len(self.edges))
        for edge in self.edges:
            write_out(f,edge[0])
            write_out(f,edge[1])
            write_out(f,edge[2])

    def serialise_load(self,f):
        read_in(f) # graph
        version=read_in(f)
        self.root=read_in(f)
        edge_count=int(read_in(f))
        self.edges=[]
        for i in range(0,edge_count):
            self.add_edge([read_in(f),
                           read_in(f),
                           float(read_in(f))])
        self.update_vertices()
 
    # clear the graph and replace it with a randomly connected
    # set of nodes - not necessarily forming a single connected 
    # compoenent graph
    def randomise(self,nverts,nedges):
        self.edges=[]
        for e in range(0,nedges):
            self.add_edge([random.randrange(0,nverts),
                           random.randrange(0,nverts),
                           float(random.random()*5)])
        self.update_vertices()
    
    # just dumps the graph to stout
    def printme(self):
        for e in self.edges:
            print (str(e[0])+" <- "+str(e[2])+" -> "+str(e[1]))
        print("root is "+str(self.root))		
		
    # check if two graphs are equivalent (ignores order of edges)
    def compare(self,other):
        if len(self.edges)!=len(other.edges):
            print ("edge sizes differ %d vs %d" % (len(self.edges),len(other.edges)))
            return False
        ret=True
        for e in self.edges:
            found=False
            for oe in other.edges:
                if self.edge_eq(oe,e) : found=True
            if found==False:
                ret=False
                print("no edge found for: %s %s %f" % (e[0],e[1],e[2]))
        return ret
	
    # returns True if the edge is eguivalent to another (ignoring direction)
    def edge_eq(self,a,b):
        if a[0]==b[0] and a[1]==b[1] or a[1]==b[0] and a[0]==b[1]:
            return True
        else:
            return False

    # a safe way to add edges - checks if it exists already
    def add_edge(self,edge):
        if self.root=="":
            self.root=edge[0]
        
        if not self.contains_edge(edge):
            self.edges.append(edge)
            self.vertices[edge[0]]=0
            self.vertices[edge[1]]=0

    # remove the edge from the graph - warning, doesn't seem to update
    # the vertices set
    def remove_edge(self,edge):
        if edge in self.edges:
            self.edges.remove(edge)
        else:
            if [edge[1],edge[0],edge[2]] in self.edges:
                self.edges.remove(edge)
            else:
                print("couldn't find edge to remove")

    # returns True if the graph contains this vertex
    def contains_vertex(self,vertex):
        return vertex in self.vertices

    # adds an unconnected vertex
    def add_vertex(self,vertex):
        self.vertices[vertex]=0

    # updates the vertices set according to the edges  
    # this will remove unconnected vertices
    def update_vertices(self):
        self.vertices = {}
        for edge in self.edges:
            self.vertices[edge[0]]=0
            self.vertices[edge[1]]=0
    
    # returns the number of vertices in the graph
    def num_vertices(self):
        return len(self.vertices)

    # returns true if the graph contains this edge
    def contains_edge(self,edge):
        for e in self.edges:
            if self.edge_eq(e,edge):
                return True
        return False

    # returns edges connected to the vertex
    def get_edges(self,vertex):
        ret = []
        for edge in self.edges:
            if edge[0]==vertex:
                ret.append(edge)
            if edge[1]==vertex:
                ret.append(edge)
        return ret

    # returns edges connected to the vertex
    def get_connected_verts(self,vertex):
        ret = {}
        for edge in self.edges:
            if edge[0]==vertex:
                ret[edge[1]]=edge[2]
            if edge[1]==vertex:
                ret[edge[0]]=edge[2]
        return ret
    
    # returns the cheapest edge in the whole graph
    def get_cheapest_edge(self):
        lowest_cost = 9999
        cheapest_edge = [-1,-1,9999]
        for edge in self.edges:
            if edge[2]<lowest_cost:
                lowest_cost=edge[2]
                cheapest_edge=edge
        return cheapest_edge

    # get the vertex in this graph with the most edges
    def get_most_connected_vert(self):
        highest_vert=-1
        highest_count=0
        for vertex in self.vertices:
            connected = self.get_connected_verts(vertex)
            if len(connected)>highest_count:
                highest_count=len(connected)
                highest_vert=vertex
        return highest_vert

    # recursive search function
    def is_connected_walk(self,vertex,history,target):
        to_search=self.get_connected_verts(vertex)
        for vertex in to_search:
            if not vertex in history:
                if vertex==target: return True
                history.append(vertex)
                if self.is_connected_walk(vertex,history,target):
                    return True
        return False

    # are the two vertices connected?
    def is_connected(self,a,b):
        return self.is_connected_walk(a,[],b)

    # returns True if the graph consists of a single
	# connected component (all vertices are joined to the root)
    def is_single_graph(self):
        for vertex in self.vertices:
            if not self.is_connected(self.root,vertex):
                return False
        return True
		
    # returns the matrix representation of the graph
    def matrix(self):
        links=[]
        highest = 0
        for edge in self.edges:
            fr=edge[0]
            to=edge[1]
            if fr>highest: highest=fr
            if to>highest: highest=to
            links.append([fr,to,float(edge[2])])
        #make the matrix
        matrix = []
        for i in range(0, highest+1):
            row=[]
            for j in range(0, highest+1):
                row.append(0)
            matrix.append(row)
        for link in links:
            matrix[link[0]][link[1]]=link[2]
            matrix[link[1]][link[0]]=link[2]
        return matrix
		
    # returns the minimum spanning tree of this graph
    # using prim's algorithm. this will fail and return False if 
    # the graph is not a single connected component, or if 
    # given a starting vertex not in the graph
    def mst(self,vertex):
        candidate_edges = graph([])
        mst = graph([])
        mst.add_vertex(vertex)
        while mst.num_vertices()<self.num_vertices():
            # get the edges from the current vertex
            for edge in self.get_edges(vertex):
                # one of the vertices in the edge must be new
                if not (mst.contains_vertex(edge[0]) and mst.contains_vertex(edge[1])):
                    # add them to the candidates for lookup
                    candidate_edges.add_edge(edge)
            
            # find and remove the cheapest edge from the candidates
            cheapest = candidate_edges.get_cheapest_edge()
            if cheapest[0]==-1 or cheapest[1]==-1: return False
            candidate_edges.remove_edge(cheapest)

            # see if this edge is needed
            if not (mst.contains_vertex(cheapest[0]) and mst.contains_vertex(cheapest[1])):
                # find out which vertex to move to
                if mst.contains_vertex(cheapest[0]): vertex=cheapest[1]
                else: vertex=cheapest[0]            
                mst.add_edge(cheapest)

        return mst

    # loads a graph from a space separated string description consisting of
    # node1 node2 strength ...
    def load(self,data):
        line = 0
        for l in data.split("\n"):
            graph=[]
            if l!="":
                t=l.split()
                if len(t)%3!=0:
                    showerror("Problem reading file", "Wrong number of things on line %d\nCarrying on anyway, but please check your file." % line)
                else:
                    for i in range(0,int(len(t)/3)):
                        c=i*3
                        fr=t[c]
                        to=t[c+1]
                        self.add_edge([fr,to,float(t[c+2])])
            line+=1
        self.root = self.get_most_connected_vert()

    # saves a space separated string description
    def save(self,filename):
        f = file(filename,"w")
        for edge in self.edges:
            f.write(str(edge[0])+" "+str(edge[1])+" "+str(edge[2])+"\n")
        f.close()
       
    # saves a graphviz dot file representation of the graph
    def save_dot(self,filename):
        f = file(filename,"w")
        f.write("graph {\n")
        for edge in self.edges:
            f.write("\""+str(edge[0])+"\" -- \""+str(edge[1]) + "\" [ label=\""+str(float(edge[2]))+"\"];\n")
        f.write("}\n")
        f.close()

    # saves a graphviz dot file including sub-nodes to represent
    # the strength of the edges, and formatted a bit nicer
    def save_hapdot(self,filename):
        f = file(filename,"w")
        f.write("graph {\n")
        #f.write("model=\"subset\"");
        #f.write("overlap=\"orthoyx\"\n")
        f.write("nodesep=20\n")
        f.write("packMode=\"node\"\n")
        nextid=0
        for vertex in self.vertices:
            f.write("\""+str(vertex)+"\" [shape=circle,width=1,height=1]\n")

        for edge in self.edges:
            if edge[2]==1:
                f.write("\""+str(edge[0])+"\" -- \""+str(edge[1])+"\"\n")
            else:
                f.write(str(edge[0])+" -- "+"i"+str(nextid)+"\n")
                for i in range(0,int(edge[2])):
                    f.write("\"i"+str(nextid)+"\" -- "+"\"i"+str(nextid+1)+"\"\n")
                    nextid=nextid+1                
                f.write("\"i"+str(nextid)+"\" -- \""+str(edge[1])+"\"\n")
                nextid=nextid+1

        for vertex in range(0,nextid):
             f.write("\"i"+str(vertex)+"\" [shape=circle,style=filled,label=\"\",width=0.3,height=0.3]\n")
            
        f.write("}\n")
        f.close()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# a simple svg document generator
class svg_doc:
	def __init__(self,w,h):
		self.doc=self.svg_header(w,h)

	def start_group(self,id,name):
		self.doc+='<g id="%s" class="node"><title>%s</title>\n'%(id,name)

	def end_group(self):
		self.doc+='</g>'

	def ellipse(self,cx,cy,rx,ry,fill):
		self.doc+='<ellipse style="fill:%s;stroke:black;" cx="%f" cy="%f" rx="%f" ry="%f"/>\n'%(fill,cx,cy,rx,ry)
		
	def text(self,x,y,size,text):
		self.doc+='<text text-anchor="middle" x="%f" y="%f" style="font-family:Times New Roman;font-size:%f;">%s</text>\n'%(x,y,size,text)
	
	def line(self,x1,y1,x2,y2):
		self.doc+='<path style="fill:none;stroke:black;" d="M %f %f L %f %f"/>\n'%(x1,y1,x2,y2)
		
	def svg_header(self,w,h):
		return """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.0//EN"
"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd" [
<!ATTLIST svg xmlns:xlink CDATA #FIXED "http://www.w3.org/1999/xlink">
]>
<!-- Made by haptree  -->
<svg width="%d" height="%d" viewBox="0.00 0.00 %d %d" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">\n"""%(w,h,w,h)

	def save(self,filename):
		f = open(filename,"w")
		f.write(self.doc+'</svg>')
		f.close();
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# some vector maths

def dst(a,b):
    x = b[0]-a[0]
    y = b[1]-a[1]
    return math.sqrt(x*x + y*y)

def dstsq(a,b):
    x = b[0]-a[0]
    y = b[1]-a[1]
    return x*x + y*y

def vmag(a):
    return math.sqrt(a[0]*a[0] + a[1]*a[1])

def vnorm(a):
    m = vmag(a)
    return [a[0]/m,a[1]/m]

def vdot(a,b):
    return a[0]*b[0] + a[1]*b[1]

def vsub(a,b):
    return [a[0]-b[0],a[1]-b[1]]

def vadd(a,b):
    return [a[0]+b[0],a[1]+b[1]]

def vmul(a,v):
    return [a[0]*v,a[1]*v]

def lerp(a,b,t):
    return [a[0]*t+b[0]*(1-t),a[1]*t+b[1]*(1-t)]

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# a force directed graph drawing class

class drawable_graph:
    def __init__(self,graph,root):
        self.type="tree"
        self.finished = 1000
        self.global_length = 30
        self.global_radius = 5
        self.text_size = 6
        self.global_strn = 0.1
        self.nodes={}
        self.graph=graph
        self.toradians = 0.0174532925
        self.root = root
        self.init_layout(self.root,[midx,midy],0,0,180)

    def serialise_save(self,f):
        version=1
        write_out(f,"drawable graph")
        write_out(f,version)
        write_out(f,self.root)
        self.graph.serialise_save(f)
        write_out(f,len(self.nodes))
        for vertex,node in self.nodes.items():
            write_out(f,vertex)
            write_out(f,node[0])
            write_out(f,node[1])

    def serialise_load(self,f):
        read_in(f) # graph
        version=read_in(f)
        self.root=read_in(f)
        self.graph.serialise_load(f)
        node_count=int(read_in(f))
        self.nodes={}
        for i in range(0,node_count):
             vertex=read_in(f)
             self.nodes[vertex]=[float(read_in(f)),
                                 float(read_in(f))]
    
    def set_type(self,type):
        self.type=type

    def random_init(self):
        print("random_init")
        for vertex in self.graph.vertices:
            self.nodes[vertex]=[random.randrange(midx-100,midx+100),
                                random.randrange(midy-100,midy+100)]

    def reinit(self):
        self.nodes={}
        if self.type=="mst":
            print("layout_init")
            self.init_layout(self.root,[midx,midy],0,0,180)
        else:
            self.random_init()
        
    def init_layout(self,vertex,ppos,dist,amin,amax):
        a = amin+(amax-amin)/2.0
        self.nodes[vertex]=vadd(ppos,vmul([math.sin(self.toradians*a),math.cos(self.toradians*a)],dist*self.global_length))
        connected = self.graph.get_connected_verts(vertex)
        outward=[]
        for k,v in connected.items():
            if not k in self.nodes:
                outward.append([k,v])

        if len(outward)>0:
            arange = amax-amin
            c=0
            apere = arange/len(outward)
            for i in outward:
                tmin=amin+c*apere
                tmax=amin+(c+1)*apere
                self.init_layout(i[0],self.nodes[vertex],i[1],tmin,tmax)
                c=c+1

    def get_intermidiaries(self):
        ret=[]
        for edge in self.graph.edges:
            for i in range(1, int(edge[2])):
                ret.append(lerp(self.nodes[edge[0]],self.nodes[edge[1]],i/float(edge[2])))
        return ret

    def process(self):
        if self.type=="mst":
            self.process_tree()
        else:
            self.process_graph()

    def process_tree(self):        
        intermidiaries = self.get_intermidiaries()

        for vertex in self.graph.vertices:
            if vertex!=self.graph.root:
                p = self.nodes[vertex]
                d = [0,0]
                connected = self.graph.get_connected_verts(vertex)
                
                for v in self.graph.vertices:
                    op=self.nodes[v]
                    strn = 0
                    if vertex!=v:
                        dist = dst(op,p)
                        if dist>0.00001:
                            if v in connected and dist>connected[v]*self.global_length:
                                strn=dist-connected[v]*self.global_length
                            else:
                                strn=1/dist*-2
                                
                            bet=vnorm(vsub(op,p))
                            # clamp to stop it exploding
                            if strn>5: strn=5;
                            if strn<-5: strn=-5;
                            strn*=self.global_strn
                            d[0]=d[0]+bet[0]*strn
                            d[1]=d[1]+bet[1]*strn

                for op in intermidiaries:
                    dist = dst(op,p)
                    if dist>0.00001:
                        bet=vnorm(vsub(op,p))
                        strn=1/dist*-1.0
                        # clamp to stop it exploding
                        if strn>5: strn=5;
                        if strn<-5: strn=-5;
                        strn*=self.global_strn
                        d[0]=d[0]+bet[0]*strn
                        d[1]=d[1]+bet[1]*strn
                
                self.nodes[vertex][0]=self.nodes[vertex][0]+d[0]
                self.nodes[vertex][1]=self.nodes[vertex][1]+d[1]

    def process_graph(self):
        for vertex in self.graph.vertices:
            p = self.nodes[vertex]
            d = [0,0]
            d[0]=(midx-p[0])*0.00001; # auto centering
            d[1]=(midy-p[1])*0.00001; 
            connected = self.graph.get_connected_verts(vertex)
            for v in self.graph.vertices:
                op=self.nodes[v]
                strn = 0
                if vertex!=v:
                    dist = dst(op,p)
                    if dist>0.00001:
                        if v in connected and dist>connected[v]*self.global_length:
                            strn=dist-connected[v]*self.global_length
                        else:
                            strn=1/dist*-2
                            
                        bet=vnorm(vsub(op,p))
                        # clamp to stop it exploding
                        if strn>5: strn=5;
                        if strn<-5: strn=-5;
                        strn*=self.global_strn*0.3
                        d[0]=d[0]+bet[0]*strn
                        d[1]=d[1]+bet[1]*strn
                                
            self.nodes[vertex][0]=self.nodes[vertex][0]+d[0]
            self.nodes[vertex][1]=self.nodes[vertex][1]+d[1]
                
    def save_svg(self,filename,w,h,zoom):
        svg=svg_doc(w,h) 
        centre=[w/2,h/2]
        for edge in self.graph.edges:
            svg.start_group("edge","edge")
            start=copy.copy(self.nodes[edge[0]])
            end=copy.copy(self.nodes[edge[1]])
		
            start[0]=centre[0]+(centre[0]-start[0])*zoom
            start[1]=centre[1]+(centre[1]-start[1])*zoom
            end[0]=centre[0]+(centre[0]-end[0])*zoom
            end[1]=centre[1]+(centre[1]-end[1])*zoom
			
            svg.line(start[0],start[1],end[0],end[1])
            for i in range(1, int(edge[2])):
                p = lerp(start,end,i/float(edge[2]))
                svg.ellipse(p[0],p[1],self.global_radius*zoom/2,self.global_radius*zoom/2,"black")
            svg.end_group()
                
        for vertex in self.graph.vertices:
            svg.start_group("vertex","vertex")
            p=copy.copy(self.nodes[vertex])
            p[0]=centre[0]+(centre[0]-p[0])*zoom
            p[1]=centre[1]+(centre[1]-p[1])*zoom
            svg.ellipse(p[0],p[1],self.global_radius*zoom,
                        self.global_radius*zoom,"white")
            svg.text(p[0],p[1]+0.5*self.global_radius*zoom,self.text_size*zoom,str(vertex))
            svg.end_group()
        svg.save(filename)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# this is probably going to be the cause of some problems but
# seems to work up to now (fingers crossed)
# ascii should be recommended though

def nasty_rtf2ascii(filename):
    f = open(filename,"r")
    d = f.read()
    f.close()  
    out=""
    ignoringctrl=False
    ignoringparen=False
    firstparent=False
    inctrl=False
    ignorenext=0
    for c in d:
        if c=="{":
            if firstparent: firstparent=True
            else: ignoringparen=True
        if c=="\\": 
            ignoringctrl=True 
            inctrl=True
        if inctrl and c=="'": 
            ignorenext=3
            ignoringctrl=False 
            inctrl=False
        if ignoringctrl and c in string.whitespace: ignoringctrl=False
        if not ignoringctrl and not ignoringparen and ignorenext<=0 and c!="}": 
            out=out+c
        if ignoringparen and c=="}": ignoringparen=False
        ignorenext=ignorenext-1
    return out

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# the gui

class win:
    def __init__(self):
        # create window
        self.root = Tk()
        self.root.title("HapStar "+version)        
        f=Frame(self.root)
        f.pack(fill=NONE);

        # make the menu
        self.root.option_add('*tearOff', FALSE)
        menubar = Menu(self.root)
        self.root['menu']=menubar
        menu_file = Menu(menubar)
        menu_about = Menu(menubar)
        menubar.add_cascade(menu=menu_file, label='File')
        menubar.add_cascade(menu=menu_about, label='About')

        menu_file.add_command(label='Import Connections', command=self.load)
        menu_file.add_command(label='Save Hapstar Network', command=self.save_hapstar)
        menu_file.add_command(label='Load Hapstar Network', command=self.load_hapstar)
        menu_file.add_command(label='Export as DOT', command=self.save_dot)
        menu_file.add_command(label='Export as SVG', command=self.save_svg)
        menu_about.add_command(label='About', command=self.on_about)

        self.run_button = Button(f, text="Run", command=self.gostop)
        self.run_button.grid(row=0, column=0, sticky="we")
        Button(f, text="Restart", command=self.restart).grid(row=0, column=1, sticky="we")

        self.mst_var = IntVar()
        cb=Checkbutton(f, text="MST", variable=self.mst_var, command=self.mst)
        cb.grid(row=0, column=2)
        cb.select()

        ef = Frame(f)
        ef.grid(row=0, column=3)
        Label(ef, text="Speed").grid(row=0,column=0)
        self.energy_scale = Scale(ef, from_=0, to=200, orient=HORIZONTAL, command=self.on_energy)
        self.energy_scale.grid(row=1, column=0)
        self.energy_scale.set(100)

        lf = Frame(f)
        lf.grid(row=0, column=4)
        Label(lf, text="Zoom").grid(row=0,column=0)
        lbf = Frame(lf)
        lbf.grid(row=1,column=0)
        Button(lbf,text="+",command=self.zoom_in).grid(row=0,column=0)
        Button(lbf,text="-",command=self.zoom_out).grid(row=0,column=1)

        #Label(lf, text="Branch length").grid(row=0,column=0)
        #self.length_scale = Scale(lf, from_=0, to=100, orient=HORIZONTAL, command=self.on_length)
        #self.length_scale.grid(row=1, column=0)
        #self.length_scale.set(30)
        
        lf = Frame(f)
        lf.grid(row=0, column=5)
        Label(lf, text="Text Size").grid(row=0,column=0)
        self.text_size_entry = Entry(lf, width=5)
        self.text_size_entry.grid(row=1, column=0)
        self.text_size_entry.bind("<Return>", self.on_text_size)
        self.text_size_entry.delete(0, END)
        self.text_size_entry.insert(0, "6")
        self.text_size=6

        rf = Frame(f)
        rf.grid(row=0, column=6)
        Label(rf, text="Circle Radius").grid(row=0,column=0)
        self.radius_entry = Entry(rf, width=5)
        self.radius_entry.grid(row=1, column=0)
        self.radius_entry.bind("<Return>", self.on_radius)
        self.radius_entry.delete(0, END)
        self.radius_entry.insert(0, "5.0")
        self.radius=5

        self.canvas=Canvas(self.root, width=500, height=480, bg='white')
        self.centre=[midx,midy]
        self.canvas.pack(fill="both", expand=1)
        self.canvas.bind("<Button-1>", self.on_canvas_click)
        self.canvas.bind("<B1-Motion>", self.on_canvas_move)
        self.canvas.bind("<B1-ButtonRelease>", self.on_release)
        self.canvas.bind("<Button-3>", self.on_canvas_right_click)
        self.canvas.bind("<B3-Motion>", self.on_canvas_right_move)

        self.dragging_node=False
        self.mouse_x=0
        self.mouse_y=0
        self.stop=1
        self.zoom=1
        self.ready=False
        #t.grid_rowconfigure(1,weight=0)
        # start event loop
        self.idle()
        self.root.mainloop()

    def idle(self):
        self.run()
        self.root.update()
        if self.stop==0: self.root.after(1,self.idle)

    def save_hapstar(self):
        filename = asksaveasfilename(filetypes = [('HPS', '*.hps')], title = "Save as hapstar document")
        if filename!="":
            f = open(filename,"w")
            version = 1
            write_out(f,version)
            write_out(f,self.mst_var.get())
            write_out(f,self.energy_scale.get())
            #pickle.dump(self.length_scale.get(),f)
            write_out(f,self.text_size)
            write_out(f,self.radius)
            write_out(f,self.zoom)
            self.mstgraph.serialise_save(f)
            self.origraph.serialise_save(f)
            f.close()

    def load_hapstar(self):
        filename = askopenfilename(filetypes = [('HPS', '*.hps')], title = "Load haplotype network text file")
        if filename!="":
            f = open(filename,"r")
            version = int(read_in(f))
            self.mst_var.set(float(read_in(f)))
            self.mst()
            self.energy_scale.set(float(read_in(f)))
            self.on_energy(0)
            #self.length_scale.set(pickle.load(f))
            #self.on_length(0)
            self.text_size=read_in(f)
            self.text_size_entry.delete(0, END)
            self.text_size_entry.insert(0, str(self.text_size))
            self.on_text_size(0)
            self.radius=read_in(f)
            self.radius_entry.delete(0, END)
            self.radius_entry.insert(0, str(self.radius))
            self.on_radius(0)
            self.zoom=float(read_in(f))
            g=graph([])
            self.mstgraph=drawable_graph(g,g.root)
            g=graph([])
            self.origraph=drawable_graph(g,g.root)
            self.mstgraph.serialise_load(f)
            self.origraph.serialise_load(f)
            if self.mst_var.get()==1:
                self.graph=self.mstgraph
            else:
                self.graph=self.origraph
            self.ready=True
            self.draw_graph(self.graph)
            f.close()

    def load(self):
        filename = askopenfilename(title = "Load haplotype network text file")
        if filename!="":
            d=""
            if os.path.splitext(filename)[1]==".rtf":
                d=nasty_rtf2ascii(filename)
            else:
                f = open(filename,"r")
                d = f.read()
                f.close()
            
            g = graph([])
            g.load(d)
            if not g.is_single_graph():
                showerror("Graph error", "The graph you have loaded consists of multiple separate, disconnected networks - this could be due to some missing connections. I can't continue.")
                return
            self.mstgraph = drawable_graph(g.mst(g.root),g.root)
            self.mstgraph.set_type("mst")
            self.origraph = drawable_graph(g,g.root)
            self.origraph.random_init()
            if self.mst_var.get()==1:
                self.graph=self.mstgraph
            else:
                self.graph=self.origraph
            self.ready=True
            self.zoom=1

        self.graph.global_strn=self.energy_scale.get()/100.0
        self.graph.text_size=self.text_size
        self.graph.global_radius=self.radius
        
        if self.stop==0:
            self.quick_draw_graph(self.graph)
        else:
            self.draw_graph(self.graph)

    def gostop(self):
        if self.ready:
            if self.stop==0:
                self.stop=1
                self.run_button.config(relief="raised")
                self.draw_graph(self.graph)
            else:
                self.stop=0
                self.run_button.config(relief="sunken")
                self.idle()

    def restart(self):
        self.graph.reinit()
        if self.stop==0:
            self.quick_draw_graph(self.graph)
        else:
            self.draw_graph(self.graph)

    def run(self):
        if self.ready and self.stop==0:
            self.graph.process()
            self.quick_draw_graph(self.graph)
        if self.ready and self.stop==1:
            self.draw_graph(self.graph)
			
    def zoom_out(self):
        self.zoom=self.zoom*0.95
        self.run()

    def zoom_in(self):
        self.zoom=self.zoom*1.05
        self.run()

    def mst(self):
        if self.ready:
            if self.mst_var.get()==1:
                self.graph=self.mstgraph
            else:
                self.graph=self.origraph
                
            self.draw_graph(self.graph)
            self.graph.global_strn=self.energy_scale.get()/100.0
            self.graph.text_size=self.text_size
            self.graph.global_radius=self.radius

    def on_energy(self,event):
        if self.ready:
            self.mstgraph.global_strn=self.energy_scale.get()/100.0
            self.origraph.global_strn=self.energy_scale.get()/100.0

    def on_length(self,event):
        if self.ready:
            self.mstgraph.global_length=self.length_scale.get()
            self.origraph.global_length=self.length_scale.get()

    def on_text_size(self,event):
        try:
            self.text_size = float(self.text_size_entry.get())
            if self.ready:
                self.mstgraph.text_size=self.text_size
                self.origraph.text_size=self.text_size
                if self.stop==1:
                    self.draw_graph(self.graph)
        except:
            pass
        
    def on_radius(self,event):
        try:
            self.radius = float(self.radius_entry.get())
            if self.ready:
                self.mstgraph.global_radius=self.radius
                self.origraph.global_radius=self.radius
            if self.stop==1:
                self.draw_graph(self.graph)
        except:
            pass
        
    def quick_draw_graph(self,graph):
        self.canvas.delete(ALL)
        for edge in graph.graph.edges:
            start=copy.copy(graph.nodes[edge[0]])
            end=copy.copy(graph.nodes[edge[1]])
            start[0]=self.centre[0]+(self.centre[0]-start[0])*self.zoom
            start[1]=self.centre[1]+(self.centre[1]-start[1])*self.zoom
            end[0]=self.centre[0]+(self.centre[0]-end[0])*self.zoom
            end[1]=self.centre[1]+(self.centre[1]-end[1])*self.zoom
            self.canvas.create_line([start[0],start[1],end[0],end[1]], fill="black")
            #if start[0]<0 or start[0]>640 or start[1]<0 or start[1]>480 or end[0]<0 or end[0]>640 or end[1]<0 or end[1]>480 :
            #    self.zoom=self.zoom*0.995 
			       
    def draw_graph(self,graph):
        self.canvas.delete(ALL)
        for edge in graph.graph.edges:
            start=copy.copy(graph.nodes[edge[0]])
            end=copy.copy(graph.nodes[edge[1]])
            start[0]=self.centre[0]+(self.centre[0]-start[0])*self.zoom
            start[1]=self.centre[1]+(self.centre[1]-start[1])*self.zoom
            end[0]=self.centre[0]+(self.centre[0]-end[0])*self.zoom
            end[1]=self.centre[1]+(self.centre[1]-end[1])*self.zoom
            self.canvas.create_line([start[0],start[1],end[0],end[1]], fill="black")
            size = int(self.graph.global_radius*self.zoom/2)
            for i in range(0, int(edge[2])):
                p = lerp(start,end,i/edge[2])
                self.canvas.create_oval([p[0]-size,p[1]-size,p[0]+size,p[1]+size], fill="black")  

        for k,ov in graph.nodes.items():
            v = copy.copy(ov)
            v[0]=self.centre[0]+(self.centre[0]-v[0])*self.zoom
            v[1]=self.centre[1]+(self.centre[1]-v[1])*self.zoom
			
            #if v[0]<0 or v[0]>640 or v[1]<0 or v[1]>480:
            #    self.zoom=self.zoom*0.95
			
            size = int(self.graph.global_radius*self.zoom)
            # need to put tag on the front of the tag name, as a number alone doesn't seem to
            # work with getitem_withtag below (which we need to move the number and the circle together)
            node = self.canvas.create_oval([v[0]-size,v[1]-size,v[0]+size,v[1]+size], fill="white", tags="tag"+str(k))
            self.canvas.tag_bind(node, '<ButtonPress-1>', self.on_click)
            self.canvas.tag_bind(node, '<B1-Motion>', self.on_drag)
            if size>2:
                node = self.canvas.create_text([v[0],v[1]], fill="black", text=str(k), tags="tag"+str(k), font=("Helvectica", str(int(self.text_size*self.zoom))))
                self.canvas.tag_bind(node, '<ButtonPress-1>', self.on_click)
                self.canvas.tag_bind(node, '<B1-Motion>', self.on_drag)
                

    def on_canvas_click(self, event):
        self.mouse_x=event.x
        self.mouse_y=event.y

    def on_canvas_move(self, event):
        if not self.dragging_node and self.ready:
            mouse_x=event.x
            mouse_y=event.y  
            dx = (self.mouse_x-mouse_x)/self.zoom
            dy = (self.mouse_y-mouse_y)/self.zoom
            for name,node in self.graph.nodes.items():
                node[0]+=dx
                node[1]+=dy

            if self.stop==0:
                self.quick_draw_graph(self.graph)
            else:
                self.draw_graph(self.graph)
                
            self.mouse_x=mouse_x
            self.mouse_y=mouse_y

    def on_canvas_right_click(self, event):
        self.mouse_x=event.x
        self.mouse_y=event.y

    def on_canvas_right_move(self, event):
        if self.ready:
            mouse_x=event.x
            mouse_y=event.y  
            dy = (self.mouse_y-mouse_y)
            self.zoom+=dy*0.01

            if self.zoom<0.1: self.zoom=0.1
            
            
            if self.stop==0:
                self.quick_draw_graph(self.graph)
            else:
                self.draw_graph(self.graph)
                
            self.mouse_x=mouse_x
            self.mouse_y=mouse_y

    def on_click(self, event):
        self.dragging_node=True
        self.curr_x = event.x
        self.curr_y = event.y

    def on_drag(self, event):        
        dx = (self.curr_x-event.x)
        dy = (self.curr_y-event.y)
        vertex=self.canvas.gettags("current")[0][3:]
        self.graph.nodes[vertex][0]+=dx/self.zoom
        self.graph.nodes[vertex][1]+=dy/self.zoom
        items = self.canvas.find_withtag("tag"+vertex)
        for item in items:
            self.canvas.move(item, -dx, -dy)
        self.curr_x = event.x
        self.curr_y = event.y

    def on_release(self, event):
        if self.ready:
            self.dragging_node=False
            if self.stop==0:
                self.quick_draw_graph(self.graph)
            else:
                self.draw_graph(self.graph)
        
    def save_svg(self):
        filename = asksaveasfilename(filetypes = [('SVG', '*.svg')], title = "Save as svg drawing")
        if filename!="":
            self.graph.save_svg(filename, 500, 480, self.zoom)

    def save_dot(self):
        filename = asksaveasfilename(filetypes = [('DOT', '*.dot')], title = "Save as dot graph")
        if filename!="":
            self.graph.graph.save_hapdot(filename)

    def save_ps(self):
        f = open("test.ps","w")
        paper_height=10
        f.write(self.canvas.postscript(fontmap='fontMap',
                   colormap='colorMap',
                   pageanchor='nw',
                   height='%2.1fc' %paper_height,
                   width='21.c',
                   pagex='0.c',
                   pagey='%2.1fc' %paper_height,
                   y='0.c', 
                   x='0.c'))
        f.close()

    def on_about(self):
        self.aboutwin=Toplevel(master=self.root)
        Label(self.aboutwin,text="HapStar v"+version+" Copyright (C) 2012 FoAM vzw\n\nBy Dave Griffiths and Amber Teacher\n\nContact: dave@fo.am",justify=LEFT).pack()
        Button(self.aboutwin,text=" Ok ",command=self.on_about_ok).pack()

    def on_about_ok(self):
        self.aboutwin.destroy()
        

w = win()

